unit CDDBConnect;

interface

uses
  IdHTTP, Classes, CDInfo;

// undefine this if you're using Indy 8
{$DEFINE INDY9}

type
  TMultipleMatchesEvent = procedure(Sender : TObject; AMatches : TStrings;
    var Index : integer; var Accept : Boolean) of object;
  TDataEvent = procedure(Sender : TObject; AData : string) of object;
  TCDDBEntry = class
    private
    protected
      FCDTitle: string;
      FArtistTitle: string;
      FTrackList : TStringList;
      FGenre: string;
      FCDID: string;
      procedure FillFromResponse(AResponse : TStrings; ACDID : string);
      function GetTracks: TStrings; virtual;
    public
      constructor Create;
      destructor Destroy; override;
      function Edit : Boolean;
      procedure FillFromCDInfo(ACDInfo : TCDInfo);
      // class functions
      class function GetItemFromResponse(AResponseLine: string;
        AIndex : integer; ReturnEntireLine : Boolean = False): string;    	
      // properties      
      property CDID : string read FCDID write FCDID;
      property CDTitle : string read FCDTitle write FCDTitle;
      property ArtistTitle : string read FArtistTitle write FArtistTitle;
      property Tracks : TStrings read GetTracks;
      property Genre : string read FGenre;  
  end;

  TCDDBConnect = class(TComponent)   
    private    	
      FOnSendData: TDataEvent;    	
      FOnReceiveData: TDataEvent;    	
      function ShowMatches(AResponse : TStrings): integer;
    protected    	
      FServer: string;      
      FClientVersion: string;      
      FClientName: string;
      FHostName: string;      
      FUserName: string;
      FHTTPConnection : TIdHTTP;      
      FProxyServer : string;      
      FProxyPort : integer;
      FLastError : integer;      
      FResponse : TStringList;      
      function GenerateCmd(CmdName: string): string;
    public      
      constructor Create (AOwner : TComponent); override;      
      destructor Destroy; override;      
      // methods      
      function SendCommand(ACmd : string) : TStrings;      
      function Help(ACmd : string) : TStrings;      
      function Sites : TStrings;      
      function MessageOfTheDay : TStrings;      
      function Stats : TStrings;      
      function GetInfoForCurrentCD(ACDEntry : TCDDBEntry) : Boolean;      
      function Users : TStrings;
      procedure SubmitEntry(ACDEntry :  TCDDBEntry);      
      // properties      
      property LastError : integer read FLastError;
    published    	
      // properties      
      property UserName : string read FUserName write FUserName;      
      property HostName : string read FHostName write FHostName;      
      property ClientName : string read FClientName write FClientName;      
      property ClientVersion : string read FClientVersion write FClientVersion;  		
      property Server : string read FServer write FServer;
      property ProxyServer : string read FProxyServer write FProxyServer;       
      property ProxyPort : integer read FProxyPort write FProxyPort;
      // events      
      property OnSendData : TDataEvent read FOnSendData write FOnSendData;      
      property OnReceiveData : TDataEvent read FOnReceiveData write FOnReceiveData;
  end;

procedure Register;

////////////////////////////////////////////////////////////////////////////////

implementation

uses  
  Forms, Controls, SysUtils, Math, Dialogs, CDDBMultipleMatchesFrm,
  CDEntryEditFrm;

const	
  CPlusSep = '+';

////////////////////////////////////////////////////////////////////////////////

procedure Register;
begin
  RegisterComponents('DoogalSoft', [TCDDBConnect]);
end;

////////////////////////////////////////////////////////////////////////////////
{ TCDDBEntry }

constructor TCDDBEntry.Create;
begin
  inherited;  
  FTrackList := TStringList.Create;
end;

////////////////////////////////////////////////////////////////////////////////

destructor TCDDBEntry.Destroy;
begin  
  FTrackList.Free;
  inherited;
end;

////////////////////////////////////////////////////////////////////////////////

function TCDDBEntry.Edit: Boolean;
var
  LCDEditFrm : TCDEntryEditForm;
  Loop : integer;
begin
  LCDEditFrm := TCDEntryEditForm.Create(nil);
  try
    LCDEditFrm.CDTitleEdit.Text := CDTitle;
    LCDEditFrm.ArtistTitleEdit.Text := ArtistTitle;
    LCDEditFrm.TracksGrid.RowCount := Tracks.Count + 1;
    for Loop := 0 to Tracks.Count-1 do
    begin
      LCDEditFrm.TracksGrid.Cells[0, Loop+1] := IntToStr(Loop+1);
      LCDEditFrm.TracksGrid.Cells[1, Loop+1] := Tracks[Loop];
    end;
    Result := (LCDEditFrm.ShowModal = mrOK);
    if Result then
    begin
      CDTitle := LCDEditFrm.CDTitleEdit.Text;
      ArtistTitle := LCDEditFrm.ArtistTitleEdit.Text;
      for Loop := 0 to Tracks.Count-1 do
      begin
        Tracks[Loop] := LCDEditFrm.TracksGrid.Cells[1, Loop+1];
      end;
    end;
  finally
    LCDEditFrm.Free;
  end;
end;

////////////////////////////////////////////////////////////////////////////////

procedure TCDDBEntry.FillFromCDInfo(ACDInfo: TCDInfo);
var
  Loop : integer;
begin
  FCDID  := ACDInfo.CDDBDiscIDHex;
  FCDTitle := 'Unknown';
  FArtistTitle := 'Unknown';
  Tracks.Clear;
  for Loop := 0 to ACDInfo.TotalTrackCount-1 do
    Tracks.Add('Track ' + IntToStr(Loop+1));
end;

////////////////////////////////////////////////////////////////////////////////

procedure TCDDBEntry.FillFromResponse(AResponse: TStrings; ACDID : string);
var
  LCount : integer;
  LTitle : string;
begin
  // get the genre
  FGenre := GetItemFromResponse(AResponse[0], 1);
  FCDID := ACDID;

  // get the artist and disc title
  LTitle := AResponse.Values['DTITLE'];
  if (Pos('/', LTitle) > 0) then
  begin
    FArtistTitle := Trim(Copy(LTitle, 1, Pos('/', LTitle)-1));
    FCDTitle := Trim(Copy(LTitle, Pos('/', LTitle)+1, Length(LTitle)));
  end
  else
  begin
    FCDTitle := LTitle;
    FArtistTitle := LTitle;
  end;

  // get the track names
  LCount := 0;
  FTrackList.Clear;
  while AResponse.Values['TTITLE' + IntToStr(LCount)] <> '' do
  begin
    FTrackList.Add(AResponse.Values['TTITLE' + IntToStr(LCount)]);
    Inc(LCount);
  end;
end;

////////////////////////////////////////////////////////////////////////////////

class function TCDDBEntry.GetItemFromResponse(AResponseLine: string;
  AIndex : integer; ReturnEntireLine : Boolean = False): string;

var
  Loop : integer;

begin
  Result := AResponseLine;
  for Loop := 1 to AIndex do
    Result := Copy(Result, Pos(' ', Result)+1, Length(Result));
  if not ReturnEntireLine then
    Result := Copy(Result, 1, Pos(' ', Result)-1);
end;

////////////////////////////////////////////////////////////////////////////////

function TCDDBEntry.GetTracks: TStrings;
begin
  Result := FTrackList;
end;

////////////////////////////////////////////////////////////////////////////////
{ TCDDBConnect }

constructor TCDDBConnect.Create(AOwner : TComponent);
begin
  inherited;
  FHTTPConnection := TIdHTTP.Create(nil);
  FUserName := 'Unknown';
  FClientVersion := '1.0';
  FClientName := 'Unknown';
  FHostName := 'Unknown';
  FServer := 'http://freedb.freedb.org';
  FResponse := TStringList.Create;
end;

////////////////////////////////////////////////////////////////////////////////

destructor TCDDBConnect.Destroy;
begin
  FResponse.Free;
  FHTTPConnection.Free;
  inherited;
end;

////////////////////////////////////////////////////////////////////////////////

function TCDDBConnect.SendCommand(ACmd: string): TStrings;
var
  LFullCmd : string;
  LMemStrm : TStringStream;
  LCmd : TStringList;
begin
  Screen.Cursor := crHourglass;
  LMemStrm := TStringStream.Create('');
  LCmd := TStringList.Create;
  try
    FResponse.Clear;
    Result := FResponse;
    LFullCmd := GenerateCmd(ACmd);
    if Assigned(FOnSendData) then
      FOnSendData(Self, LFullCmd);
    LCmd.Text := LFullCmd;
    {$IFDEF INDY9}
    FHTTPConnection.ProxyParams.ProxyServer := FProxyServer;
    FHTTPConnection.ProxyParams.ProxyPort := FProxyPort;
    FHTTPConnection.HTTPOptions := FHTTPConnection.HTTPOptions-[hoForceEncodeParams];
    {$ELSE}
    FHTTPConnection.Request.ProxyServer := FProxyServer;
    FHTTPConnection.Request.ProxyPort := FProxyPort;
    {$ENDIF}
    FHTTPConnection.Post(FServer + '/~cddb/cddb.cgi', LCmd, LMemStrm);
    LMemStrm.Position := 0;
    Result.Text := LMemStrm.DataString;
    FLastError := StrToIntDef(Copy(Result.Text, 1, 3), 0);
  finally
    FHTTPConnection.Disconnect;
    if Assigned(FOnReceiveData) then
      FOnReceiveData(Self, LMemStrm.DataString);
    LCmd.Free;
    LMemStrm.Free;
    Screen.Cursor := crDefault;
  end;
end;

////////////////////////////////////////////////////////////////////////////////

function TCDDBConnect.GenerateCmd(CmdName : string) : string;
begin
  Result := 'cmd=' + CmdName +
    '&hello=' + FUserName + CPlusSep + FHostName + CPlusSep + FClientName + CPlusSep
    + FClientVersion + '&proto=4';
end;

////////////////////////////////////////////////////////////////////////////////

function TCDDBConnect.Help(ACmd: string): TStrings;
begin
  Result := SendCommand('help' + CPlusSep + ACmd);
end;

////////////////////////////////////////////////////////////////////////////////

function TCDDBConnect.Sites: TStrings;
begin
  Result := SendCommand('sites');
end;

////////////////////////////////////////////////////////////////////////////////

function TCDDBConnect.MessageOfTheDay: TStrings;
begin
  Result := SendCommand('motd');
end;

////////////////////////////////////////////////////////////////////////////////

function TCDDBConnect.Stats: TStrings;
begin
  Result := SendCommand('stat');
end;

////////////////////////////////////////////////////////////////////////////////

function TCDDBConnect.ShowMatches(AResponse : TStrings) : integer;
var
  Loop : integer;
  LMultipleMatches : TCDDBMultipleMatchesForm;

begin
  LMultipleMatches := TCDDBMultipleMatchesForm.Create(Self);
  Loop := 1;
  while AResponse[Loop] <> '.' do
  begin
    LMultipleMatches.MatchesList.Items.Add(
      TCDDBEntry.GetItemFromResponse(AResponse[Loop], 2, True));
    Inc(Loop);
  end;
  Result := -1;
  if LMultipleMatches.ShowModal = mrOK then
    Result := LMultipleMatches.MatchesList.ItemIndex;
  LMultipleMatches.Free;
end;

////////////////////////////////////////////////////////////////////////////////

function TCDDBConnect.GetInfoForCurrentCD(ACDEntry : TCDDBEntry) : Boolean;
var
  LCDInfo : TCDInfo;
  LCmd : string;
  Loop : integer;
  LResponse : TStrings;

  procedure DoRead(AInfo : string);
  begin
    // get the CD info
    LResponse := SendCommand('cddb' + CPlusSep + 'read' + CPlusSep + AInfo);
    if LastError = 210 then
    begin
      Result := True;
      ACDEntry.FillFromResponse(LResponse, LCDInfo.CDDBDiscIdHex);
    end;
  end;

  procedure HandleMultipleMatches;
  var
    LMatchIndex : integer;
  begin
    LMatchIndex := ShowMatches(LResponse);
    if LMatchIndex > -1 then
      DoRead(TCDDBEntry.GetItemFromResponse(LResponse[LMatchIndex+1], 0) +
      	CPlusSep + TCDDBEntry.GetItemFromResponse(LResponse[LMatchIndex+1], 1));
  end;

  procedure HandleUnknownDisc;
  begin
    ACDEntry.FillFromCDInfo(LCDInfo);
    if ACDEntry.Edit then
    begin
      SubmitEntry(ACDEntry);
      Result := True;
    end;
  end;

begin
  Result := False;
  LCDInfo := TCDInfo.Create(nil);
  try
    // query the database first
    LCmd := 'cddb' + CPlusSep + 'query' + CPlusSep + LCDInfo.CDDBDiscIdHex +
      '+' + IntToStr(LCDInfo.TotalTrackCount);
    for Loop := 1 to LCDInfo.TotalTrackCount do
      LCmd := LCmd + CPlusSep + IntToStr(LCDInfo.TOCToFrames(LCDInfo.TrackPosition(Loop)));
    LCmd := LCmd + CPlusSep + IntToStr(LCDInfo.TotalCDLength);
    LResponse := SendCommand(LCmd);

    case LastError of
      200 : DoRead(TCDDBEntry.GetItemFromResponse(LResponse[0], 1) +
    	      CPlusSep + LCDInfo.CDDBDiscIdHex);
      // TODO - allow adding CDs to CDDB
      202 : HandleUnknownDisc;
      210, 211 : HandleMultipleMatches;
    else
      raise Exception.Create('Unexpected response - ' + LResponse.Text);
    end;
  finally
    LCDInfo.Free;
  end;
end;

////////////////////////////////////////////////////////////////////////////////

function TCDDBConnect.Users: TStrings;
begin
  Result := SendCommand('whom');
end;

procedure TCDDBConnect.SubmitEntry(ACDEntry: TCDDBEntry);
begin

end;

end.