unit Rptbtn;

interface

uses
  Classes,
{$IFDEF MSWINDOWS}
  Buttons, ExtCtrls, Controls;
{$ENDIF}
{$IFDEF LINUX}
  QButtons, QExtCtrls, QControls;
{$ENDIF}

type
  TRepeatButton = class(TSpeedButton)
  private
    { Private declarations }
    Timer : TTimer;
    FRepeatTime : Word;
    FOnClick : TNotifyEvent;
    Repeated : Boolean;
    procedure SetRptTime (Value : Word);
    procedure OnTimer (Sender : TObject);
  protected
    { Protected declarations }
  public
    { Public declarations }
    constructor Create (AOwner : TComponent); override;
    destructor Destroy; override;
    procedure Click; override;
    procedure MouseUp(Button : TMousebutton; Shift : TShiftState; X, Y : integer); override;
  	procedure MouseDown(Button : TMousebutton; Shift : TShiftState; X, Y : integer); override;
  published
    { Published declarations }
    property RepeatTime : Word read FRepeatTime write SetRptTime default 50;
    property OnClick : TNotifyEvent read FOnClick write FOnClick;
  end;

procedure Register;

implementation

////////////////////////////////////////////////////////////////////////////////

procedure Register;
begin
  RegisterComponents('DoogalSoft', [TRepeatButton]);
end;

{******************************************************************************}

constructor TRepeatButton.Create (AOwner : TComponent);

begin
  inherited Create(AOwner);
  Timer := TTimer.Create(Self);
  Timer.Enabled := False;
  Timer.OnTimer := OnTimer;
  RepeatTime := 50;
  Repeated := False;
end;

{******************************************************************************}

destructor TRepeatButton.Destroy;

begin
	Timer.Free;
  inherited Destroy;
end;

{******************************************************************************}

procedure TRepeatButton.MouseDown(Button : TMousebutton; Shift : TShiftState; X, Y : integer);

begin
	inherited MouseDown(Button, Shift, X, Y);
  Timer.Enabled := True;
end;

{******************************************************************************}

procedure TRepeatButton.MouseUp(Button : TMousebutton; Shift : TShiftState; X, Y : integer);
begin
	inherited MouseUp(Button, Shift, X, Y);
	Timer.Enabled := False;
end;

{******************************************************************************}

procedure TRepeatButton.OnTimer (Sender : TObject);

begin
  if Assigned (FOnClick) then
		FOnClick(Self);
  Repeated := True;
end;

{******************************************************************************}

procedure TRepeatButton.SetRptTime (Value : Word);

begin
  FRepeatTime := Value;
  if Assigned(Timer) then
  	Timer.Interval := Value;
end;

{******************************************************************************}

procedure TRepeatButton.Click;
begin
	inherited Click;
  if Assigned (FOnClick) then
  if not Repeated then
   	FOnClick(Self);
  Repeated := False;
end;

end.
